"use strict";

//Commonly used Tests tool code for reuse in Step-By-Step documents

//elements for updating results of tests
var statusElement;
var runningElement;
var passedElement;
var failedElement;
var boldElement;
var logElement;
var lastmessage;

//Run a test by name, hook up output elements using an id naming convention.
function runTest(test){

  if (tests.isBusy()){
    alert("Test is busy");
    return;
  }

  var id = test.replace(/ /g, '_'); //global replacement, spaces to underscores
/*
statusElement: Div which contains all "old" test messages from the current test.
runningElement: Div which is visible only while test is running.
boldElement: Div which is meant to contain only the most recent test message.
logElement:Div which contains the test readout data, bold and status..
passedElement: Div which displays when passed. This is used to hide
  the next tests from the user until the first test passes.
failedElement: Div which displays when the test fails. Certain
  failure specific data is also displayed, see tests.OnStatusMessage below.
Each of these divs must be found within html files where the test tool is used.
*/
  runningElement = elementFromID(id + "_running");
  statusElement = elementFromID(id + "_status");
  boldElement = elementFromID(id+ "_bold");
  logElement = elementFromID(id+ "_log");
  passedElement = elementFromID(id + "_passed");
  failedElement = elementFromID(id + "_failed");

  clear(statusElement);
  clear(boldElement);
  clear(failedElement);
  setVisible(logElement);
  setVisible(statusElement);
  setVisible(runningElement);
  setVisible(passedElement, false);
  setVisible(failedElement);
  scrollTo(statusElement);
  lastmessage=undefined; //clear out lastmessage incase it has a value.
  tests.runTest(test, coms.axis()); //start the test
}


onGDKLoad( function() {
  coms.onComsConnected(testsConnected); //react to future changes in connection
  testsConnected(coms.isConnected()); //may have entered the page connected, so reac
  tests.onBurnBusy(testsBurnBusy);
  tests.onStatusMessage( function(message) {
  	if (lastmessage != "Test Failed") {
    if (lastmessage != undefined){ //If there is a non undefined value...
        print(statusElement, lastmessage,true);//Move the current bold item to log.
      }
    print(boldElement, message, false);//
    scrollTo(failedElement);
	} else if (lastmessage == "Test Failed") {
		print(statusElement, message, true)
	}
    console.log(message);
    if (message==='Amplifier Error'){ //If the error is a specific case I care about..
      console.log("AMPERR");
      setVisible('amp_err');
    }
    if (message === 'Under Voltage'){
        console.log("UNDERVOLT");
        setVisible('under_voltage');
        scrollTo('under_voltage');
        setVisible('amp_err',false);
    }
    if (message === 'Electronic Lock Out (ELO)'){
      setVisible('elo');
      scrollTo('elo');
      setVisible('amp_err',false);
    }
    if (message === 'Hall Error'){
        setVisible('hall_err');
        scrollTo('hall_err');
        setVisible('amp_err',false);
    }
    lastmessage=message;//Now that the parsing is done, save the current message.
  });

  tests.onTestResult( function(result) {
    console.log(result);
    if (lastmessage != undefined){
        print(statusElement, lastmessage,true);
      }
    if (result > 0){//If the test passed..
      print(boldElement,"Test Passed", false);
      setVisible(amp_err,false);
    } else if (result < 0){//If the test failed..
      print(boldElement,"Test Failed",false);
      print(boldElement,"Please check your wiring and retry",true);
      print(boldElement,"If errors persist, contact application support",true);
    }
    //Things to do regardless of failed/passsed result.
    setVisible(runningElement,false);
    setVisible(passedElement, result > 0);
    setVisible(failedElement, result < 0);
    scrollTo(statusElement,result>0);
    lastmessage="Test Failed"
  });
});

function testsConnected(connected){//Runs when a controller connects/disconnects.
  if (connected){
    tests.create('stack', false); //opens tool behind others in stack if one is not open.
  }
}

function setDirection(direction){
    coms.command('CE%m='+direction, function(command, response){
      console.log(direction);
      if(direction===0){
        print('Motor_Direction_bold', 'Normal Polarity Set');
      }else{
        print('Motor_Direction_bold', 'Reversed Polarity Set');
      }
    });
    setVisible('Motor_Direction_passed',true);
    setVisible('Motor_Direction_log',true);
    scrollTo('Motor_Direction_passed');
  }

function setPolarity(direction){
      console.log(direction);
      if(direction===1){
        coms.command('MT%m=-2', function(command, response){  });
        print('Motor_Direction_bold', 'Normal Polarity Set');
      }else{
        coms.command('MT%m=-2.5', function(command, response){  });
        print('Motor_Direction_bold', 'Reversed Polarity Set');
      }
    setVisible('Motor_Direction_passed',true);
    setVisible('Motor_Direction_log',true);
    scrollTo('Motor_Direction_passed');
  }

function testsBurnBusy(busy){
  setVisible(save_running,busy);
  setVisible(save_passed,!busy);
  if (busy){
    print(save_bold,'Step 1/1: Saving results to controller...',true);
    scrollTo(save_log);
  } else if (!busy){
    print(save_status,"Step 1/1: Saving results to controller...",true);
    print(save_bold,"Test results saved.",false);
    setVisible(save_bold);
    scrollTo(save_passed);
  }
}

function saveResults(){
  clear(save_bold);
  clear(save_status);
  setVisible(save_log);
  tests.downloadAndBurn();
}


function isLicensed(license){
  console.log('license:',license.valid);
  if(license.valid){
    setVisible('license');
  } else {
    setVisible('no_license');
  }
}

function noLicense(){
  alert('Your version of GDK is not licensed, and so does not support the autotuning tools. Contact Galil via the link at the bottom of the page to purchase a license.')
    }

  function scrollTest(target){
    var targetElement = target; //assume we've already found the DOM element
    if (typeof target === "string"){ //nope, we have the id
      targetElement = document.getElementById(target);
    }
    if (targetElement == null){ //null or undefined
      return;
    }

    targetElement.scrollIntoView();
  }

  function serialReturn(axis, features){
    setVisible('serial', features.ssi || features.biss);
  }

  function serialIndividual(axis, features){
    setVisible('ssi', features.ssi);
    setVisible('biss',features.biss);
  }
